@@ -12,6 +12,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import os
+
 import mock
 from oslo_concurrency import processutils
 from oslotest import base as test_base
@@ -26,26 +28,27 @@ else:
     OPEN_FUNCTION_NAME = 'builtins.open'
 
 
+def _build_fake_image_info():
+    return {
+        'id': 'fake_id',
+        'urls': [
+            'http://example.org',
+        ],
+        'checksum': 'abc123'
+    }
+
+
 class TestStandbyExtension(test_base.BaseTestCase):
     def setUp(self):
         super(TestStandbyExtension, self).setUp()
         self.agent_extension = standby.StandbyExtension()
 
-    def _build_fake_image_info(self):
-        return {
-            'id': 'fake_id',
-            'urls': [
-                'http://example.org',
-            ],
-            'checksum': 'abc123'
-        }
-
     def test_validate_image_info_success(self):
-        standby._validate_image_info(None, self._build_fake_image_info())
+        standby._validate_image_info(None, _build_fake_image_info())
 
     def test_validate_image_info_missing_field(self):
         for field in ['id', 'urls', 'checksum']:
-            invalid_info = self._build_fake_image_info()
+            invalid_info = _build_fake_image_info()
             del invalid_info[field]
 
             self.assertRaises(errors.InvalidCommandParamsError,
@@ -53,7 +56,7 @@ class TestStandbyExtension(test_base.BaseTestCase):
                               invalid_info)
 
     def test_validate_image_info_invalid_urls(self):
-        invalid_info = self._build_fake_image_info()
+        invalid_info = _build_fake_image_info()
         invalid_info['urls'] = 'this_is_not_a_list'
 
         self.assertRaises(errors.InvalidCommandParamsError,
@@ -61,7 +64,7 @@ class TestStandbyExtension(test_base.BaseTestCase):
                           invalid_info)
 
     def test_validate_image_info_empty_urls(self):
-        invalid_info = self._build_fake_image_info()
+        invalid_info = _build_fake_image_info()
         invalid_info['urls'] = []
 
         self.assertRaises(errors.InvalidCommandParamsError,
@@ -69,7 +72,7 @@ class TestStandbyExtension(test_base.BaseTestCase):
                           invalid_info)
 
     def test_validate_image_info_invalid_checksum(self):
-        invalid_info = self._build_fake_image_info()
+        invalid_info = _build_fake_image_info()
         invalid_info['checksum'] = {'not': 'a string'}
 
         self.assertRaises(errors.InvalidCommandParamsError,
@@ -77,7 +80,7 @@ class TestStandbyExtension(test_base.BaseTestCase):
                           invalid_info)
 
     def test_validate_image_info_empty_checksum(self):
-        invalid_info = self._build_fake_image_info()
+        invalid_info = _build_fake_image_info()
         invalid_info['checksum'] = ''
 
         self.assertRaises(errors.InvalidCommandParamsError,
@@ -90,14 +93,14 @@ class TestStandbyExtension(test_base.BaseTestCase):
                           image_info={'foo': 'bar'})
 
     def test_image_location(self):
-        image_info = self._build_fake_image_info()
+        image_info = _build_fake_image_info()
         location = standby._image_location(image_info)
         self.assertEqual(location, '/tmp/fake_id')
 
     @mock.patch(OPEN_FUNCTION_NAME, autospec=True)
     @mock.patch('ironic_python_agent.utils.execute', autospec=True)
     def test_write_image(self, execute_mock, open_mock):
-        image_info = self._build_fake_image_info()
+        image_info = _build_fake_image_info()
         device = '/dev/sda'
         location = standby._image_location(image_info)
         script = standby._path_to_script('shell/write_image.sh')
@@ -200,7 +203,7 @@ class TestStandbyExtension(test_base.BaseTestCase):
     @mock.patch(OPEN_FUNCTION_NAME)
     @mock.patch('requests.get')
     def test_download_image(self, requests_mock, open_mock, md5_mock):
-        image_info = self._build_fake_image_info()
+        image_info = _build_fake_image_info()
         response = requests_mock.return_value
         response.status_code = 200
         response.iter_content.return_value = ['some', 'content']
@@ -212,7 +215,37 @@ class TestStandbyExtension(test_base.BaseTestCase):
 
         standby._download_image(image_info)
         requests_mock.assert_called_once_with(image_info['urls'][0],
-                                              stream=True)
+                                              stream=True, proxies={})
+        write = file_mock.write
+        write.assert_any_call('some')
+        write.assert_any_call('content')
+        self.assertEqual(write.call_count, 2)
+
+    @mock.patch('hashlib.md5')
+    @mock.patch(OPEN_FUNCTION_NAME)
+    @mock.patch('requests.get')
+    @mock.patch.dict(os.environ, {})
+    def test_download_image_proxy(
+            self, requests_mock, open_mock, md5_mock):
+        image_info = _build_fake_image_info()
+        proxies = {'http': 'http://a.b.com',
+                   'https': 'https://secure.a.b.com'}
+        no_proxy = '.example.org,.b.com'
+        image_info['proxies'] = proxies
+        image_info['no_proxy'] = no_proxy
+        response = requests_mock.return_value
+        response.status_code = 200
+        response.iter_content.return_value = ['some', 'content']
+        file_mock = mock.Mock()
+        open_mock.return_value.__enter__.return_value = file_mock
+        file_mock.read.return_value = None
+        hexdigest_mock = md5_mock.return_value.hexdigest
+        hexdigest_mock.return_value = image_info['checksum']
+
+        standby._download_image(image_info)
+        self.assertEqual(no_proxy, os.environ['no_proxy'])
+        requests_mock.assert_called_once_with(image_info['urls'][0],
+                                              stream=True, proxies=proxies)
         write = file_mock.write
         write.assert_any_call('some')
         write.assert_any_call('content')
@@ -220,57 +253,40 @@ class TestStandbyExtension(test_base.BaseTestCase):
 
     @mock.patch('requests.get', autospec=True)
     def test_download_image_bad_status(self, requests_mock):
-        image_info = self._build_fake_image_info()
+        image_info = _build_fake_image_info()
         response = requests_mock.return_value
         response.status_code = 404
         self.assertRaises(errors.ImageDownloadError,
                           standby._download_image,
                           image_info)
 
-    @mock.patch('ironic_python_agent.extensions.standby._verify_image',
-                autospec=True)
+    @mock.patch('hashlib.md5', autospec=True)
     @mock.patch(OPEN_FUNCTION_NAME, autospec=True)
     @mock.patch('requests.get', autospec=True)
     def test_download_image_verify_fails(self, requests_mock, open_mock,
-                                         verify_mock):
-        image_info = self._build_fake_image_info()
+                                         md5_mock):
+        image_info = _build_fake_image_info()
         response = requests_mock.return_value
         response.status_code = 200
-        verify_mock.side_effect = errors.ImageChecksumError(
-            'foo', '/bar/foo', 'incorrect', 'correct')
+        hexdigest_mock = md5_mock.return_value.hexdigest
+        hexdigest_mock.return_value = 'invalid-checksum'
         self.assertRaises(errors.ImageChecksumError,
                           standby._download_image,
                           image_info)
 
-    @mock.patch(OPEN_FUNCTION_NAME)
-    @mock.patch('hashlib.md5')
-    def test_verify_image_success(self, md5_mock, open_mock):
-        image_info = self._build_fake_image_info()
-        hexdigest_mock = md5_mock.return_value.hexdigest
-        hexdigest_mock.return_value = image_info['checksum']
-        file_mock = mock.Mock()
-        file_mock.read.return_value = None
-        open_mock.return_value.__enter__.return_value = file_mock
+    def test_verify_image_success(self):
+        image_info = _build_fake_image_info()
         image_location = '/foo/bar'
+        checksum = image_info['checksum']
+        standby._verify_image(image_info, image_location, checksum)
 
-        verified = standby._verify_image(image_info, image_location)
-        self.assertTrue(verified)
-        self.assertEqual(1, md5_mock.call_count)
-
-    @mock.patch(OPEN_FUNCTION_NAME)
-    @mock.patch('hashlib.md5')
-    def test_verify_image_failure(self, md5_mock, open_mock):
-        image_info = self._build_fake_image_info()
-        md5_mock.return_value.hexdigest.return_value = 'wrong hash'
-        file_mock = mock.Mock()
-        file_mock.read.return_value = None
-        open_mock.return_value.__enter__.return_value = file_mock
+    def test_verify_image_failure(self):
+        image_info = _build_fake_image_info()
         image_location = '/foo/bar'
-
+        checksum = 'invalid-checksum'
         self.assertRaises(errors.ImageChecksumError,
                           standby._verify_image,
-                          image_info, image_location)
-        self.assertEqual(md5_mock.call_count, 1)
+                          image_info, image_location, checksum)
 
     @mock.patch('ironic_python_agent.hardware.dispatch_to_managers',
                 autospec=True)
@@ -280,7 +296,7 @@ class TestStandbyExtension(test_base.BaseTestCase):
                 autospec=True)
     def test_cache_image(self, download_mock, write_mock,
                          dispatch_mock):
-        image_info = self._build_fake_image_info()
+        image_info = _build_fake_image_info()
         download_mock.return_value = None
         write_mock.return_value = None
         dispatch_mock.return_value = 'manager'
@@ -305,7 +321,7 @@ class TestStandbyExtension(test_base.BaseTestCase):
                 autospec=True)
     def test_cache_image_force(self, download_mock, write_mock,
                                dispatch_mock):
-        image_info = self._build_fake_image_info()
+        image_info = _build_fake_image_info()
         self.agent_extension.cached_image_id = image_info['id']
         download_mock.return_value = None
         write_mock.return_value = None
@@ -333,7 +349,7 @@ class TestStandbyExtension(test_base.BaseTestCase):
                 autospec=True)
     def test_cache_image_cached(self, download_mock, write_mock,
                                 dispatch_mock):
-        image_info = self._build_fake_image_info()
+        image_info = _build_fake_image_info()
         self.agent_extension.cached_image_id = image_info['id']
         download_mock.return_value = None
         write_mock.return_value = None
@@ -368,7 +384,7 @@ class TestStandbyExtension(test_base.BaseTestCase):
                            write_mock,
                            dispatch_mock,
                            configdrive_copy_mock):
-        image_info = self._build_fake_image_info()
+        image_info = _build_fake_image_info()
         location_mock.return_value = '/tmp/configdrive'
         download_mock.return_value = None
         write_mock.return_value = None
@@ -428,7 +444,7 @@ class TestStandbyExtension(test_base.BaseTestCase):
                                           write_mock,
                                           dispatch_mock,
                                           configdrive_copy_mock):
-        image_info = self._build_fake_image_info()
+        image_info = _build_fake_image_info()
         download_mock.return_value = None
         write_mock.return_value = None
         dispatch_mock.return_value = 'manager'
@@ -451,6 +467,52 @@ class TestStandbyExtension(test_base.BaseTestCase):
                       ).format(image_info['id'], 'manager')
         self.assertEqual(cmd_result, async_result.command_result['result'])
 
+    @mock.patch(('ironic_python_agent.extensions.standby.'
+                 '_write_configdrive_to_partition'),
+                autospec=True)
+    @mock.patch('ironic_python_agent.hardware.dispatch_to_managers',
+                autospec=True)
+    @mock.patch('ironic_python_agent.extensions.standby.StandbyExtension'
+                '._cache_and_write_image', autospec=True)
+    @mock.patch('ironic_python_agent.extensions.standby.StandbyExtension'
+                '._stream_raw_image_onto_device', autospec=True)
+    def _test_prepare_image_raw(self, image_info, stream_mock,
+                                cache_write_mock, dispatch_mock,
+                                configdrive_copy_mock):
+        dispatch_mock.return_value = '/dev/foo'
+        configdrive_copy_mock.return_value = None
+
+        async_result = self.agent_extension.prepare_image(
+            image_info=image_info,
+            configdrive=None
+        )
+        async_result.join()
+
+        dispatch_mock.assert_called_once_with('get_os_install_device')
+        self.assertFalse(configdrive_copy_mock.called)
+
+        # Assert we've streamed the image or not
+        if image_info['stream_raw_images']:
+            stream_mock.assert_called_once_with(mock.ANY, image_info,
+                                                '/dev/foo')
+            self.assertFalse(cache_write_mock.called)
+        else:
+            cache_write_mock.assert_called_once_with(mock.ANY, image_info,
+                                                     '/dev/foo')
+            self.assertFalse(stream_mock.called)
+
+    def test_prepare_image_raw_stream_true(self):
+        image_info = _build_fake_image_info()
+        image_info['disk_format'] = 'raw'
+        image_info['stream_raw_images'] = True
+        self._test_prepare_image_raw(image_info)
+
+    def test_prepare_image_raw_and_stream_false(self):
+        image_info = _build_fake_image_info()
+        image_info['disk_format'] = 'raw'
+        image_info['stream_raw_images'] = False
+        self._test_prepare_image_raw(image_info)
+
     @mock.patch('ironic_python_agent.utils.execute', autospec=True)
     def test_run_image(self, execute_mock):
         script = standby._path_to_script('shell/shutdown.sh')
@@ -498,3 +560,79 @@ class TestStandbyExtension(test_base.BaseTestCase):
 
         execute_mock.assert_called_once_with(*command, check_exit_code=[0])
         self.assertEqual('FAILED', failed_result.command_status)
+
+    @mock.patch('ironic_python_agent.extensions.standby._write_image',
+                autospec=True)
+    @mock.patch('ironic_python_agent.extensions.standby._download_image',
+                autospec=True)
+    def test_cache_and_write_image(self, download_mock, write_mock):
+        image_info = _build_fake_image_info()
+        device = '/dev/foo'
+        self.agent_extension._cache_and_write_image(image_info, device)
+        download_mock.assert_called_once_with(image_info)
+        write_mock.assert_called_once_with(image_info, device)
+
+    @mock.patch('hashlib.md5')
+    @mock.patch(OPEN_FUNCTION_NAME)
+    @mock.patch('requests.get')
+    def test_stream_raw_image_onto_device(self, requests_mock, open_mock,
+                                          md5_mock):
+        image_info = _build_fake_image_info()
+        response = requests_mock.return_value
+        response.status_code = 200
+        response.iter_content.return_value = ['some', 'content']
+        file_mock = mock.Mock()
+        open_mock.return_value.__enter__.return_value = file_mock
+        file_mock.read.return_value = None
+        hexdigest_mock = md5_mock.return_value.hexdigest
+        hexdigest_mock.return_value = image_info['checksum']
+
+        self.agent_extension._stream_raw_image_onto_device(image_info,
+                                                           '/dev/foo')
+        requests_mock.assert_called_once_with(image_info['urls'][0],
+                                              stream=True, proxies={})
+        expected_calls = [mock.call('some'), mock.call('content')]
+        file_mock.write.assert_has_calls(expected_calls)
+
+    @mock.patch('hashlib.md5')
+    @mock.patch(OPEN_FUNCTION_NAME)
+    @mock.patch('requests.get')
+    def test_stream_raw_image_onto_device_write_error(self, requests_mock,
+                                                      open_mock, md5_mock):
+        image_info = _build_fake_image_info()
+        response = requests_mock.return_value
+        response.status_code = 200
+        response.iter_content.return_value = ['some', 'content']
+        file_mock = mock.Mock()
+        open_mock.return_value.__enter__.return_value = file_mock
+        file_mock.write.side_effect = Exception('Surprise!!!1!')
+        hexdigest_mock = md5_mock.return_value.hexdigest
+        hexdigest_mock.return_value = image_info['checksum']
+
+        self.assertRaises(errors.ImageDownloadError,
+                          self.agent_extension._stream_raw_image_onto_device,
+                          image_info, '/dev/foo')
+        requests_mock.assert_called_once_with(image_info['urls'][0],
+                                              stream=True, proxies={})
+        # Assert write was only called once and failed!
+        file_mock.write.assert_called_once_with('some')
+
+
+class TestImageDownload(test_base.BaseTestCase):
+
+    @mock.patch('hashlib.md5', autospec=True)
+    @mock.patch('requests.get', autospec=True)
+    def test_download_image(self, requests_mock, md5_mock):
+        content = ['SpongeBob', 'SquarePants']
+        response = requests_mock.return_value
+        response.status_code = 200
+        response.iter_content.return_value = content
+
+        image_info = _build_fake_image_info()
+        md5_mock.return_value.hexdigest.return_value = image_info['checksum']
+        image_download = standby.ImageDownload(image_info)
+
+        self.assertEqual(content, list(image_download))
+        requests_mock.assert_called_once_with(image_info['urls'][0],
+                                              stream=True, proxies={})
+        self.assertEqual(image_info['checksum'], image_download.md5sum())
